/*-
 * AvpMonitor for Linux (v0.1) 
 * Thanks Markus Wolf <klog@hert.org>, Promisc Security, who gives
 * us basic idea of interceptor.
 *
 * Copyright (C) 1999 Kaspersky Lab.
 * http://www.avp.ru
 *
 * This file is part of AvpMonitor
 * 
 * AvpMonitor is free software; you can redistribute it and/or modify
 * it without limitation.
 *
 * AvpMonitor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 */

#include <linux/types.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>

#include <asm/uaccess.h>
#include <asm/segment.h>
#include <asm/io.h>

extern char *auditBuf;
extern int fl_audit_open;
extern unsigned long audit_size;
//extern struct wait_queue *audit_wait;

inline void init_audit_buffer(void);
inline void release_audit_buffer(void);
asmlinkage int sys_audit(char *buf, int count);

//static int audit_read(struct inode * inode,struct file * file,char * buf,int count)
static ssize_t audit_read(struct file * file,char * buf,size_t count,loff_t *ppos)
 {
  if((file->f_flags & O_NONBLOCK) && !audit_size) return -EAGAIN;
  return sys_audit(buf,count);
 }

//static int audit_select(struct inode *inode, struct file *file, int sel_type, select_table *wait)
// {
//  if(sel_type!=SEL_IN) return 0;
//  if(audit_size) return 1;
//  select_wait(&audit_wait,wait);
//  return 0;
// }

static int proc_audit_unlink(struct inode *dir, struct dentry *dentry)
 {
  return 0;
 }

static int audit_open(struct inode *inode, struct file *file)
 {
  if(fl_audit_open) return -EAGAIN;
  fl_audit_open++;
  if(!auditBuf) init_audit_buffer();
  return 0;
 }

static int audit_release(struct inode *inode, struct file *file)
 {
  if(!fl_audit_open) return -EAGAIN;
  fl_audit_open--;
  if(auditBuf) release_audit_buffer();
  return 0;
 }

static struct file_operations proc_audit_operations=
 {
  NULL,		// lseek   
  audit_read,	// read	   
  NULL,		// write   
  NULL,		// readdir 
  NULL,//audit_select,	// select default 
  NULL,		// ioctl   
  NULL,		// mmap	   
  audit_open,	// no special open code	   
  NULL,		// no special flush code 
  audit_release,// no special release code 
  NULL,		// fsync 
  NULL,		// fasync 
  NULL,		// check_media_change 
  NULL		// revalidate 
 };

struct inode_operations proc_audit_inode_operations =
 {
  &proc_audit_operations,// default base directory file-ops 
  NULL,			// create
  NULL,			// lookup
  NULL,			// link 
  proc_audit_unlink,	// unlink(struct inode *, struct dentry *) 
  NULL,			// symlink
  NULL,			// mkdir 
  NULL,			// rmdir 
  NULL,			// mknod 
  NULL,			// rename 
  NULL,			// readlink 
  NULL,			// follow_link 
  NULL,			// readpage
  NULL,			// writepage 
  NULL,			// bmap 
  NULL,			// truncate 
  NULL			// permission 
 };

