/*-
 * AvpMonitor for Linux/FreeBSD (v0.1) 
 * Thanks Markus Wolf <klog@hert.org>, Promisc Security, who gives
 * us basic idea of interceptor.
 *
 * Copyright (C) 1999 Kaspersky Lab.
 * http://www.avp.ru
 *
 * This file is part of AvpMonitor
 * 
 * AvpMonitor is free software; you can redistribute it and/or modify
 * it without limitation.
 *
 * AvpMonitor is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU CC; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

//#include <linux/audit.h>
#include "common.h" 
#include "../audit.h" 
#include "profile.h"
#include "report.h"

ULONG reportLimSize=0; 

int reportInit(char *log,char *LogName) //prf_data.CommonLog,prf_data.CommonLogFileName
 {
  struct stat statbuf;
  char   LogPath[0x200];
  int    hLogFile=-1;  
  ULONG  NewPointer=0; 
  time_t Timer;
  struct tm *Tm;

  Timer =time((time_t*)NULL);
  Tm=gmtime(&Timer);

  if(log)
   {
    if(LogName[0]=='/')  strcpy(LogPath,LogName);
    else
     {
      strcpy(LogPath,prg_path);
      if(LogPath[strlen(LogPath)-1]!='/') strcat(LogPath,"/");
      if((LogName[0]=='.')&&(LogName[1]=='/'))
       strcat(LogPath,&LogName[2]);
      else
       strcat(LogPath,LogName);
     } 
    
    if(LogPath[strlen(LogPath)-1]=='/') LogPath[strlen(LogPath)-1]=0;

    if((hLogFile=open(LogPath,
      (prf_data.Append)?(O_WRONLY|O_CREAT):(O_WRONLY|O_CREAT|O_TRUNC),
      S_IRUSR|S_IRGRP|S_IROTH|S_IWRITE))!=(-1))
     {
      if(fstat(hLogFile,&statbuf)) NewPointer=statbuf.st_size;
      if(prf_data.LogFileLimit) //&& (statbuf.st_size/1024)>tPage3.m_Size)
       {
        reportLimSize=atol(prf_data.LogFileSize)*1024;
        if(reportLimSize<NewPointer)
	 {
	  lseek(hLogFile,reportLimSize,SEEK_SET);
	  write(hLogFile,"",1);
	 } 
       }
      if(prf_data.Append) lseek(hLogFile,0,SEEK_END);

      char s[0x200]; // Antiviral Toolkit Pro by Eugene Kaspersky
#ifdef linux
      sprintf(s,"\r\n   AVP for Linux(Monitor) start %02d-%02d-%02d %02d:%02d:%02d\r\n\r\n",
#else
      sprintf(s,"\r\n   AVP for FreeBSD(Monitor) start %02d-%02d-%02d %02d:%02d:%02d\r\n\r\n",
#endif      
           Tm->tm_mday,Tm->tm_mon,Tm->tm_year+1900,Tm->tm_hour,Tm->tm_min,Tm->tm_sec);
      reportPutS(*log,hLogFile,s);
      return hLogFile;
     }
    else
     {
      *log=0;
      return -1;
     }
   }
  return -1;
 }

void reportPutS(char log,int hlog,char *str)
 {
  if((log)&&(hlog!=(-1))) //while(*str) reportPutC(*str++);
   {
    struct stat statbuf;
    ULONG  NewPointer=0;    
    if(fstat(hlog,&statbuf)) NewPointer=statbuf.st_size;
    if((reportLimSize>(NewPointer+strlen(str)))||(reportLimSize==0))
     {
      write(hlog,str,strlen(str));
      NewPointer+=strlen(str);
     }
    else
     {
      ULONG Dlin;
      Dlin=reportLimSize-NewPointer;
      if(Dlin<strlen(str))
       {
        write(hlog,str,Dlin);
        NewPointer+=Dlin;
       }
      else
       {
        write(hlog,str,strlen(str));
        NewPointer+=strlen(str);
       }
     }
   }
 }

void reportDone(char log,int hlog)
 {
  if((log)&&(hlog!=-1))
   close(hlog);
 }

int getunit(char *src, char *dst, int len);

void log_socket(char *info,char log,int hlog,char *buf,char *type,char *link)
 {
  if(hlog==-1) return;

  char tmp[20];
  int sport, dport;
  struct in_addr source, dest;
  char *sip, *dip, saddr[20], daddr[20];

  source.s_addr = getunit(buf, tmp, sizeof(tmp));
  sport = ntohs(getunit(buf, tmp, sizeof(tmp)));
  sip = inet_ntoa(source); strcpy(saddr, sip);

  char s[0x200];
  if(link != NULL)
   {
    dest.s_addr = getunit(buf, tmp, sizeof(tmp));
    dport = ntohs(getunit(buf, tmp, sizeof(tmp)));
    dip = inet_ntoa(dest); strcpy(daddr, dip);

    //fprintf(fd,"%s\t%s(saddr:sport %s:%d link: %s daddr:dport %s:%d)\n",info,type,
    sprintf(s,"%s\t%s(saddr:sport %s:%d link: %s daddr:dport %s:%d)\n",info,type,
        saddr, sport, link, daddr, dport);
   }
  else 
    sprintf(s,"%s\t%s(saddr:sport %s:%d)\n",info,type,saddr, sport);
  //  fprintf(fd,"%s\t%s(saddr:sport %s:%d)\n",info,type,saddr, sport);
  //  fflush(fd);
  reportPutS(log,hlog,s);
 }

void log_file(char *info,char log,int hlog,char *buf,char *type)
 {
  if(hlog==-1) return;
  //  char filename[64];

  //  getunit(buf, filename, sizeof(filename));
  //  fprintf(fd,"%s\t%s\t(\"%s\")\n",info,type,buf);//filename);
  //  fflush(fd);
  char s[0x200];
  sprintf(s,"%s\t%s\t(\"%s\")\n",info,type,buf);//filename);
  reportPutS(log,hlog,s);
 }

void log_setuid(char *info,char log,int hlog,char *buf,char *type)
 { 
  if(hlog==-1) return;
  int ruid, euid;
  char tmp[20];

  ruid = getunit(buf, tmp, sizeof(tmp));
  euid = getunit(buf, tmp, sizeof(tmp));

  //fprintf(fd,"%s\t%s(ruid: %d, euid: %d)\n",info,type, ruid, euid);
  //fflush(fd);
  char s[0x200];
  sprintf(s,"%s\t%s(ruid: %d, euid: %d)\n",info,type, ruid, euid);
  reportPutS(log,hlog,s);
 }
