#include <dirent.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <paths.h>
#include <errno.h>
#include "debug.h"
#include "check.h"
#include "shmem.h"

char Symlinks=1,Directories=1,SubDirectories=1,CrossFs=1;

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

#ifndef USESHMEM
static int ifin(char symb,char *mask)
 {
  for(int i=0;mask[i]!=0;i++)
   if(symb==mask[i]) return 1;
  return 0;
 }
 
// Return string of value. Mask some symbols.
char* masksymbols(char *str,char *mask)
 {
  if((strlen(str)==0) || (strlen(mask)==0)) return NULL;

  int kilsymb=0;
  for(int i=0;str[i]!=0;i++) if(ifin(str[i],mask) || (str[i]=='\\')) kilsymb++;

  char *targstr=(char*)malloc(strlen(str)+kilsymb+10);
  char *to=targstr;
  
  for(;*str;str++)
   {
    if(ifin(*str,mask) || (*str=='\\'))
     {
      *to='\\';
      to++;
     }
    *to=*str;
    to++;
   }     

  *to=0;
  return targstr;
 }
#endif

static int substr(char *s,char *old,char *news)
 {
  char *tmp=NULL;
  int oldlen=strlen(old),newlen=0;

  if (NULL==strstr(s, old)) return 0;

  if(news)newlen=strlen(news);

  if(newlen>oldlen)
   {
    if((tmp=(char*)malloc(strlen(s)))==NULL)
     {
      fprintf(stderr, "no memory\n");
      return 1; // exit (1);
     }
   }

  while(NULL != (s = strstr(s, old)))
   {
    char *p,*old_s=s;

    if(news)
     {
      if(newlen > oldlen)
      old_s = strcpy(tmp, s);
      p=news;
      while (*p) *s++ = *p++;
     }
    p=old_s + oldlen;
    while ((*s++ = *p++));
   }
  if(tmp) free(tmp);
  return 1;
 }

static int TidyPath (char *path)
 {
  int tidied=0;
  char *s, *p;

  strcat(path,"/");	// tmp trailing slash simplifies things 
  while(substr(path,"/./","/")) tidied = 1;
  while(substr(path,"//" ,"/")) tidied = 1;

  while (NULL != (p = strstr(path,"/../")))
   {
    for (s = p+3; p != path && *--p != '/';);
    if (*p != '/') break;
    if (p == path) ++p;
    while ((*p++ = *s++));
    tidied = 1;
   }
  if (!*path) strcpy(path,"/");
  p = path + strlen(path) - 1;
  if(p != path && *p == '/') *p-- = '\0'; // remove tmp trailing slash 
  while (p != path && *p == '/')
   {	/* remove any others */
    *p-- = '\0';
    tidied = 1;
   }
  while (!strncmp(path,"./",2))
   {
    for (p = path, s = path+2; (*p++ = *s++););
    tidied = 1;
   }
  return tidied;
 }

static char* CheckSymlink(char *path,dev_t my_dev)
 {
  static char slpath[PATH_MAX],abspath[PATH_MAX]; //,news[PATH_MAX]
  char *p;//, *lp, *tail, *msg,*np;
  struct stat stbuf;
  int c;//, fix_abs = 0, fix_messy = 0, fix_long = 0;

  if((c=readlink(path,slpath,sizeof(slpath))) == -1)
   {
    perror(path);
    return NULL;
   }
  slpath[c]='\0';	// readlink does not null terminate it

  // construct the absolute address of the link
  abspath[0]='\0';
  if(slpath[0] != '/')
   {
    strcat(abspath,path);
    if(((c=strlen(abspath)) > 0) && (abspath[c-1] == '/'))
	abspath[c-1] = '\0';
    if((p=strrchr(abspath,'/')) != NULL) *p='\0';
    strcat(abspath,"/");
   }
  strcat(abspath,slpath);
  TidyPath(abspath);

  // check for various things 
  if(stat(abspath,&stbuf)==-1)
   {
    return NULL;
   }
  return abspath;
 }

static int ScanFile(char *fullFileName,int flags)
 {
  int ret=0;
#ifdef USESHMEM
  printf("\nCurrent object: %s\n",fullFileName);
  ret=AvpTest(flags,fullFileName);
#else
  char* maskedname=masksymbols(fullFileName,";:,");
  if(maskedname!=NULL)
   {
    printf("\nCurrent object: %s(Masked: %s)\n",fullFileName,maskedname);
    ret=AvpTest(flags,maskedname);
    free(maskedname);
   }
#endif      
  return ret;
 }

static int ScanTree(char *CurrDir,int flags)
 {
  int ret=0;
  printf("Current object(directory): %s\n",CurrDir);
  ret=AvpTest(flags,CurrDir);
  return ret;
 }

int WorkByType(char *szObjName,int flags)
 {
  if(strcmp(szObjName,"-")==0)
    return ScanFile(szObjName,S_IFREG);

  struct stat st;
  if(lstat(szObjName,&st)==-1)
   {
    dprintf(("Cannot fihd object (%s). LStat()=-1\n",szObjName));
    return -1;
   }    
  dprintf(("Trying to check type of test object %s \n",szObjName));
  if(S_ISLNK(st.st_mode))
   {
    if((Symlinks==1)||(Symlinks==2))
     {                                        
      char *TargetPath=CheckSymlink(szObjName,st.st_dev);
      dprintf(("Type: symlink %s -> %s\n",szObjName,TargetPath));
      if(TargetPath!=NULL)
       if(lstat(TargetPath,&st)!=-1)
        if(S_ISDIR(st.st_mode))
	 {
          dprintf(("Type: directory. Trying to test directory %s from symlink %s \n",TargetPath,szObjName));
          if(Directories==1) return ScanTree(TargetPath,S_IFDIR);
	  else return 0;
	 }
        else
         {
          if(S_ISREG(st.st_mode) )
	   {
  	    dprintf(("Type: regular file. Trying to test file %s from symlink %s \n",TargetPath,szObjName));
            return ScanFile(TargetPath,S_IFREG);
	   }    
 	 }
     } 
   }  
  else
    if(S_ISDIR(st.st_mode))
     {
      dprintf(("Type: directory. Trying to test %s \n",szObjName));
      if(Directories==1) return ScanTree(szObjName,S_IFDIR); 
      else return 0;
     }
    else
     {
      if(S_ISREG(st.st_mode) )
       {
        dprintf(("Type: regular file. Trying to test %s \n",szObjName));
        return ScanFile(szObjName,S_IFREG);
       }	
     }
  return -1;
 }
